if select(6, GetAddOnInfo("PitBull_" .. (debugstack():match("[d%.][d%.][O%.]ns\\PitBull\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 34018 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Mana = PitBull:NewModule("Mana", "AceEvent-2.0", "AceConsole-2.0")
local self = PitBull_Mana
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-28 22:20:34 -0400 (Sat, 28 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = PitBull:L("PitBull-Mana")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

local UnitMana = UnitMana
local UnitManaMax = UnitManaMax
local UnitPowerType = UnitPowerType
local ManaBarColor = ManaBarColor

PitBull_Mana:RegisterPitBullChildFrames('manaBar')

local unitsShown = {}

function PitBull_Mana:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Mana")
	PitBull:RegisterDefaults("Mana", "profile", {
		['**'] = {
			deficit = false,
			reverse = false,
			hidden = false,
		},
		targettarget = {
			hidden = true,
		},
		targettargettarget = {
			hidden = true,
		},
	})
end

function PitBull_Mana:OnEnable(first)
	self:RegisterBucketEvent({"UNIT_MANA", "UNIT_RAGE", "UNIT_FOCUS", "UNIT_ENERGY", "UNIT_MAXMANA", "UNIT_MAXRAGE", "UNIT_MAXFOCUS", "UNIT_MAXENERGY" }, 0.05, "UNIT_MANA")
	self:RegisterBucketEvent("UNIT_DISPLAYPOWER", 0.05)
end

function PitBull_Mana:UNIT_MANA(units)
	for unit in pairs(units) do
		for frame in PitBull:IterateUnitFramesForUnit(unit) do
			if unitsShown[frame] then
				self:UpdateMana(unit, frame)
			end
		end
	end
end

function PitBull_Mana:UNIT_DISPLAYPOWER(units)
	for unit in pairs(units) do
		for frame in PitBull:IterateUnitFramesForUnit(unit) do
			if unitsShown[frame] then
				self:UpdateMana(unit, frame)
				self:UpdatePowerColor(unit, frame)
			end
		end
	end
end

function PitBull_Mana:UpdateMana(unit, frame)
	local db = self.db.profile[frame.group]
	
	local max = UnitManaMax(unit)
	if max > 0 then
		local value = UnitMana(unit)
		local perc = value / max
		if db.deficit ~= db.reverse then
			perc = 1 - perc
		end
		frame.manaBar:SetValue(perc)
	else	
		if db.deficit ~= db.reverse then
			frame.manaBar:SetValue(1)
		else
			frame.manaBar:SetValue(0)
		end
	end
end

function PitBull_Mana:UpdatePowerColor(unit, frame)
	local db = self.db.profile[frame.group]
	
	local powerType = UnitPowerType(unit)
	local r,g,b
	if powerType == 0 then
		r,g,b = unpack(PitBull.colorConstants.mana)
	elseif powerType == 1 then
		if UnitManaMax(unit) == 0 then
			r,g,b = 0.2, 0.2, 0.2
		else
			r,g,b = unpack(PitBull.colorConstants.rage)
		end
	elseif powerType == 2 then
		r,g,b = unpack(PitBull.colorConstants.focus)
	elseif powerType == 3 then
		r,g,b = unpack(PitBull.colorConstants.energy)
	else
		r,g,b = unpack(PitBull.colorConstants.unknown)
	end
	
	if db.reverse then
		frame.manaBar:SetStatusBarColor((r+0.2)/3, (g+0.2)/3, (b+0.2)/3, 1)
		frame.manaBar.bg:SetVertexColor(r, g, b, 1)
	else
		frame.manaBar:SetStatusBarColor(r, g, b, 1)
		frame.manaBar.bg:SetVertexColor((r+0.2)/3, (g+0.2)/3, (b+0.2)/3, 1)
	end
end

function PitBull_Mana:OnUpdateFrame(unit, frame)
	if frame.manaBar and unitsShown[frame] then
		self:UpdateMana(unit, frame)
		self:UpdatePowerColor(unit, frame)
	end
end

function PitBull_Mana:OnPopulateUnitFrame(unit, frame)
	if frame.manaBar or PitBull_Mana.db.profile[frame.group].hidden then
		return
	end
	local manaBar = newFrame("StatusBar", frame)
	frame.manaBar = manaBar
	manaBar:SetMinMaxValues(0, 1)
	manaBar:SetValue(1)
	manaBar:SetStatusBarTexture(PitBull:GetStatusBarTexture())
	manaBar:SetStatusBarColor(0, 0, 1, 1)

	local manaBarBG = newFrame("Texture", manaBar, "BACKGROUND")
	frame.manaBar.bg = manaBarBG
	manaBarBG:SetTexture(PitBull:GetStatusBarTexture())
	manaBarBG:SetVertexColor(0, 0, 0.5, 1)
	manaBarBG:ClearAllPoints()
	manaBarBG:SetAllPoints(manaBar)
end

function PitBull_Mana:OnClearUnitFrame(unit, frame)
	if frame.manaBar then
		frame.manaBar.bg = delFrame(frame.manaBar.bg)
		frame.manaBar = delFrame(frame.manaBar)
	end
end

function PitBull_Mana:OnUpdateStatusBarTexture(texture)
	for _,frame in PitBull:IterateUnitFrames() do
		if frame.manaBar then
			frame.manaBar:SetStatusBarTexture(texture)
			frame.manaBar.bg:SetTexture(texture)
		end
	end
end

function PitBull_Mana:OnUnknownLayout(unit, frame, name)
	frame.manaBar:SetPoint("TOPLEFT", frame, "LEFT", 5, 0)
	frame.manaBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, 5)
end

function PitBull_Mana:OnUpdateLayout(unit, frame)
	local old = not unitsShown[frame]
	local now = not frame.manaBar or not frame.manaBar:IsShown()
	if old == now then
		return
	end
	unitsShown[frame] = not now
	if not now then
		self:OnUpdateFrame(unit, frame)
	end
end

local function getDeficit(group)
	return PitBull_Mana.db.profile[group].deficit
end
local function setDeficit(group, value)
	PitBull_Mana.db.profile[group].deficit = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Mana:UpdateMana(unit, frame)
	end
end
local function getReverse(group)
	return PitBull_Mana.db.profile[group].reverse
end
local function setReverse(group, value)
	PitBull_Mana.db.profile[group].reverse = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Mana:UpdateMana(unit, frame)
		PitBull_Mana:UpdatePowerColor(unit, frame)
	end
end
local function getHide(group)
	return PitBull_Mana.db.profile[group].hidden
end
local function setHide(group, value)
	PitBull_Mana.db.profile[group].hidden = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		if value then
			self:OnClearUnitFrame(unit, frame)
		else
			self:OnPopulateUnitFrame(unit, frame)
			self:OnUpdateFrame(unit, frame)
		end
		PitBull:UpdateLayout(frame)
	end
end
PitBull_Mana:RegisterPitBullOptionsMethod(function(group)
	return {
		name = L["Mana bar"],
		desc = L["Mana display settings for this unit type."],
		type = 'group',
		args = {
			deficit = {
				type = 'toggle',
				name = L["Show as deficit"],
				desc = L["Show mana deficit, filling the bar when you use up your mana."],
				get = getDeficit,
				set = setDeficit,
				passValue = group,
				disabled = getHide,
			},
			reverse = {
				type = 'toggle',
				name = L["Reverse"],
				desc = L["Reverses the direction of the mana bar, filling it from right to left."],
				get = getReverse,
				set = setReverse,
				passValue = group,
				disabled = getHide,
			},
			hide = {
				type = 'toggle',
				name = L["Hide"],
				desc = L["Hides the mana bar completely."],
				get = getHide,
				set = setHide,
				passValue = group,
			}
		}
	}
end)
